(game-module "wreckr-t"
    (blurb "Terrain types for the Wreckreation game.")
)

#| Terrain Definitions
    Author: Eric McDonald
    Used in the Wreckreation game.
|#

; Liquid "Terrain"

(terrain-type ocean (name "ocean"))
(terrain-type sea-coast (name "sea coast"))
(terrain-type lake (name "lake"))

; Solid Terrain

(terrain-type salt-marsh (name "salt-marsh"))
(terrain-type swamp (name "swamp"))
(terrain-type bog (name "bog"))
(terrain-type jungle (name "jungle"))
(terrain-type forest (name "forest"))
(terrain-type tanglewood (name "tanglewood"))
(terrain-type hill-forest (name "forested hills"))
(terrain-type mtn-forest (name "forested mountains"))
(terrain-type hill (name "hills"))
(terrain-type mountain (name "mountains"))
(terrain-type glacier (name "glacier"))
(terrain-type tundra (name "tundra"))
(terrain-type desert (name "desert"))
(terrain-type steppe (name "steppe"))
(terrain-type green (name "green"))
(terrain-type silt-plain (name "silted plain"))

; Borders
(terrain-type river (name "river"))
(terrain-type shoreline (name "shore"))
(terrain-type cliff (name "cliffs"))
(terrain-type hedge (name "hedge"))
(terrain-type sheer-mtn (name "impassable mountains"))
(terrain-type great-wall (name "great wall"))

; Connectors
(terrain-type dirt-road (name "dirt road"))
(terrain-type paved-road (name "paved road"))

; Special Terrain
(terrain-type arena-wall (name "arena wall"))
(terrain-type arena-border (name "arena border"))

;; Terrain Groups
; GROUP

; Group by Solidity and Moisture
(define liquid-t* (ocean sea-coast lake))
(define soggy-t* (salt-marsh swamp bog))
(define woody-t* (jungle forest tanglewood hill-forest mtn-forest))
(define arid-t* (tundra desert)) 
(define semi-arid-t* (hill mountain steppe))
(define fertile-t* (green silt-plain))
(define solid-t* (append soggy-t* woody-t* arid-t* semi-arid-t* fertile-t*))

; Group by Smoothness
(define flat-t* 
    (salt-marsh swamp bog jungle forest tanglewood tundra desert steppe green 
     silt-plain)
)
(define elevated-t* (hill-forest mtn-forest hill mountain glacier))

; Group by Permissiveness
(define border-t* (river shoreline cliff hedge sheer-mtn great-wall))
(define road-t* (dirt-road paved-road))
(define all-unit-impassable-t* (arena-wall arena-border))
(define land-unit-impassable-t* (append 
    liquid-t* tanglewood hedge sheer-mtn great-wall all-unit-impassable-t*)
)
(define sea-unit-impassable-t* (append 
    solid-t* border-t* road-t* all-unit-impassable-t*)
)

; Group by Likeness
(define shallows-t* (sea-coast lake))
(define hill-t* (hill-forest hill))
(define mountain-t* (mtn-forest mountain))

; Group by Other
(define artificial-t* (hedge great-wall paved-road))

;; Terrain Identifier Characters
; CHAR

(add liquid-t* char ".")
(add soggy-t* char ",")
(add woody-t* char "%")
(add flat-t* char " ")
(add elevated-t* char "^")
(add river char ".")
(add shoreline char " ")
(add cliff char "^")
(add hedge char "%")
(add sheer-mtn char "^")
(add great-wall char "|")
(add road-t* char "_")
(add arena-wall char "|")
(add arena-border char "|")

;; Terrain Help Blurbs
; HELP

(add ocean help "Deep saltwater sea.")
(add sea-coast help "Shallow saltwater sea.")
(add lake help "Freshwater lake.")
(add salt-marsh help "Sea level saltwater marsh.")
(add swamp help "Freshwater swamp.")
(add bog help "Muddy, slimy region.")
(add jungle help "Rain forest found in warm climates.")
(add forest help "Forest found in moderate or subarctic climates.")
(add tanglewood help "Impenetrable thickets.")
(add hill-forest help "Forested hills.")
(add mtn-forest help "Forested mountains.")
(add hill help "Barren hills.")
(add mountain help "Barren mountains.")
(add glacier help "Glaciated mountains or hills.")
(add tundra help "Cold, barren plain.")
(add desert help "Barren plain found in warm climates.")
(add steppe help "Dry, grassy and shrubbed plain.")
(add green help "Green, fertile plain.")
(add silt-plain help "Frequently flooded, riverine plain. Very fertile.")
(add river help "River flowing between terrain cells.")
(add shoreline help "Beaches between water and terrain cells.")
(add cliff help "Cliffs between water and terrain cells.")
(add hedge help "Impassable thicket of woods separating terrain cells.")
(add sheer-mtn help "Impassable precipes and ledges separating terrain cells.")
(add great-wall help "Long wall separating terrain cells.")
(add dirt-road help "Crude dirt road connecting terrain cells.")
(add paved-road help "Paved road connecting terrain cells.")
(add arena-wall help "Completely impassable arena border.")
(add arena-border help "Completely impassable arena border.")

;; Terrain Images
; IMAGE

(add ocean image-name "civ-ocean")
(add sea-coast image-name "shallows")
(add lake image-name "shallows")
(add salt-marsh image-name "flooded")
(add swamp image-name "civ-swamp")
(add bog image-name "wooded-hills")
(add jungle image-name "civ-jungle")
(add forest image-name "adv-forest")
(add tanglewood image-name "civ-forest")
; (add hill-forest image-name "hill")
; (add hill-forest image-name "civ-hills")
(add hill-forest image-name "adv-forest")
; (add mtn-forest image-name "hill")
; (add mtn-forest image-name "civ-hills")
(add mtn-forest image-name "adv-forest")
(add hill image-name "adv-mountains")
; (add mountain image-name "adv-mountains")
; (add mountain image-name "civ-mountains")
(add mountain image-name "mountains")
(add glacier image-name "adv-ice")
(add tundra image-name "civ-tundra")
(add desert image-name "adv-desert")
(add steppe image-name "adv-steppe")
(add green image-name "adv-plain")
(add silt-plain image-name "civ-lush")
(add river image-name "river")
; (add shoreline image-name "beach")
(add shoreline image-name "dark-sea-green")
(add cliff image-name "cliffs")
; (add hedge image-name "light-woods")
(add hedge image-name "adv-forest")
(add sheer-mtn image-name "cliffs")
(add great-wall image-name "walltown")
(add dirt-road image-name "dirt-road")
(add paved-road image-name "flagstone")
; (add paved-road image-name "road")
(add arena-wall image-name "black")
(add arena-border image-name "dark-orange")

;; Terrain Attributes
; ATTRIB

(add liquid-t* liquid true)

(add border-t* subtype border)
(add road-t* subtype connection)
(add arena-border subtype border)

(add river subtype-x river-x)
(add dirt-road subtype-x road-x)

(set edge-terrain arena-wall)

;; Terrain Interspersion Effects
; INTERSPERSION

(table terrain-density 0
    (ocean hill-t* 25)
    (ocean mountain-t* 5)
)

;; Terrain Adjacency Effects
; ADJACENCY

(table adjacent-terrain-effect non-terrain
    (ocean mountain-t* hill-t*)
    (ocean hill-t* sea-coast)
    (ocean flat-t* sea-coast)
    (sea-coast hill-t* (green steppe))
    (lake (salt-marsh ocean sea-coast) salt-marsh)
    (salt-marsh (ocean sea-coast) sea-coast)
    (salt-marsh (swamp bog) (swamp bog))
    (salt-marsh (jungle forest) bog)
    (salt-marsh mountain-t* mountain-t*)
    ((salt-marsh swamp) glacier mountain)
    ((swamp bog) lake lake)
    (swamp bog bog)
    ((swamp bog) jungle jungle)
    ((swamp bog) forest forest)
    (hill-t* mountain-t* mountain-t*)
    (hill-forest glacier hill)
    (hill-forest (hill mountain) hill)
    (mtn-forest glacier mountain)
    (mtn-forest (hill mountain) mountain)
    (glacier (hill mountain) (hill mountain))
)
(table adjacent-terrain-effect-chance 0%
    (ocean mountain-t* 95%)
    (ocean hill-t* 75%)
    (ocean flat-t* 50%)
    (sea-coast hill-t* 25%)
    (lake (salt-marsh ocean sea-coast) 100%)
    (salt-marsh (ocean sea-coast) 75%)
    (salt-marsh (swamp bog) 100%)
    (salt-marsh (jungle forest) 100%)
    (salt-marsh mountain-t* 50%)
    ((salt-marsh swamp) glacier 100%)
    ((swamp bog) lake 100%)
    (swamp bog 50%)
    ((swamp bog) jungle 50%)
    ((swamp bog) forest 50%)
    (hill-t* mountain-t* 25%)
    (hill-forest glacier 100%)
    (hill-forest (hill mountain) 5%)
    (mtn-forest glacier 100%)
    (mtn-forest (hill mountain) 5%)
    (glacier (hill mountain) 50%)
)
(table adjacent-terrain-effect-passes -1
    (ocean mountain-t* 4)
    (ocean hill-t* 4)
    (ocean flat-t* 1)
    (sea-coast hill-t* 5)
    (salt-marsh (ocean sea-coast) 4)
    (salt-marsh mountain-t* 5)
    ((swamp bog) lake 3)
    (swamp bog 4)
    ((swamp bog) jungle 3)
    ((swamp bog) forest 3)
    (hill-t* mountain-t* 1)
    (hill-forest (hill mountain) 3)
    (mtn-forest (hill mountain) 3)
    (glacier (hill mountain) 3)
)

(table adjacent-terrain-border non-terrain
    (shallows-t* flat-t* shoreline)
    (shallows-t* elevated-t* shoreline)
    (liquid-t* soggy-t* non-terrain)
    (liquid-t* glacier non-terrain)
    (arena-wall (remove arena-wall t*) arena-border)
)
(table adjacent-terrain-border-chance 0%
    (shallows-t* flat-t* 50%)
    (shallows-t* elevated-t* 5%)
    (liquid-t* soggy-t* 0%)
    (liquid-t* glacier 0%)
    (arena-wall (remove arena-wall t*) 100%)
)

;; Elevations
; ELEVATION

; Elevations are in meters.
(add liquid-t* elevation-min 0)
(add liquid-t* elevation-max 0)
(add (salt-marsh swamp bog) elevation-min 1)
(add (salt-marsh swamp bog) elevation-max 2)
(add (tundra desert steppe green) elevation-min 3)
(add (tundra desert steppe green) elevation-max 500)
(add (jungle forest) elevation-min 5)
(add (jungle forest) elevation-max 1000)
(add (hill hill-forest) elevation-min 500)
(add (hill hill-forest) elevation-max 1500)
(add (mtn-forest mountain glacier) elevation-min 1500)
(add (mtn-forest) elevation-max 2000)
(add (mountain) elevation-max 3750)
(add (glacier) elevation-max 15000)

;; Rivers
; RIVER

(set river-sink-terrain lake)

(add liquid-t* river-chance 0)
(add (salt-marsh swamp bog) river-chance 2000)
(add jungle river-chance 3000)
(add forest river-chance 2000)
(add hill-forest river-chance 3000)
(add mtn-forest river-chance 4000)
(add hill river-chance 1000)
(add mountain river-chance 3000)
(add glacier river-chance 5000)
(add (tundra desert steppe) river-chance 0)
(add green river-chance 200)
(add silt-plain river-chance 0)
